package net.minecraft.src;


public class BLDREntityBrickBuilder extends BLDREntityBuilder{
	
	public BLDREntityBrickBuilder(World world) {
		super(world);
		buildCount = BLDRBuilderConfig.brickStructureLimit;
		health = 30;
		maxWait = 100;
		setTexture(rand.nextInt(5));
		if(rand.nextInt(10) < 9) //90% chance of having Brick Blocks
			buildBlock = BLDRBuilderConfig.brickCommonBlock;
		else //10% chance of having stone brick
			buildBlock = BLDRBuilderConfig.brickRareBlock;
		
		if(BLDRBlueprints.singlePrints.size() > 0)
			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
		else
			blueNum = -1;
		while(BLDRBlueprints.singlePrints.size() > 2 && (BLDRBlueprints.isSmallShopIndex(blueNum) || BLDRBlueprints.isSingleBlockIndex(blueNum)))
			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());

        tradeGiveNum = tradeGiveNum - rand.nextInt(3);
        tradeReceiveNum = rand.nextInt(8);
        tradeRatio = tradeRatio - rand.nextInt(8);
	}
	
	protected void initConvert(int x, String s){
		if(x == Item.brick.shiftedIndex){
			builderFlags |= (1 << 5); //pays rent three times as often
			moveSpeed += 0.2D;
			health += 5;
		}
		else if(x == Block.brick.blockID){
			builderFlags |= (1 << 4); //builds twice as fast
			health += 10;
		}
    }
	
	public boolean getCanSpawnHere()
    {
		if(worldObj.isDaytime()){
			if(mod_Builders.countBuilders(this.getClass()) >= BLDRBuilderConfig.brickSpawnMaxDay)
				return false;
			else
				return rand.nextInt(100) < BLDRBuilderConfig.brickSpawnRateDay && super.getCanSpawnHere();
		}
		else{
			if(mod_Builders.countBuilders(this.getClass()) >= BLDRBuilderConfig.brickSpawnMaxNight)
				return false;
			else
				return rand.nextInt(100) < BLDRBuilderConfig.brickSpawnRateNight && super.getCanSpawnHere();
		}
    }
	
    public void writeEntityToNBT(NBTTagCompound nbttagcompound)
    {
        super.writeEntityToNBT(nbttagcompound);
    }

    public void readEntityFromNBT(NBTTagCompound nbttagcompound)
    {
        super.readEntityFromNBT(nbttagcompound);
    }
    
    
    public void onUpdate()
    {
    	if(currentAction >= 1 && currentAction <= 6 && actionTimer == 10099 && ((builderFlags >> 1) & 1) != 1){
    		if(BLDRBlueprints.singlePrints.size() > 0)
    			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
    		else
    			blueNum = -1;
    		while(BLDRBlueprints.singlePrints.size() > 2 && (BLDRBlueprints.isSmallShopIndex(blueNum) || BLDRBlueprints.isSingleBlockIndex(blueNum)))
    			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
    	}
    	else if(currentAction == 0 && updateTimer % 10 == 0){
    		if(claySpot == null){
    			claySpot = spotClay();
    		}
    	}
    	
    	if(claySpot != null && mod_Builders.minecraft_b.thePlayer.getDistanceToEntity(this) < 8){
    		int blockDist = MathHelper.floor_double(mod_Builders.minecraft_b.thePlayer.getDistance(claySpot.xCoord, claySpot.yCoord, claySpot.zCoord));
    		String str = (new StringBuilder().append("2<").append(this.name)).append("> 1There is clay about ").append(blockDist).append(" meters from where you're standing.").toString();
    		ModLoader.getMinecraftInstance().ingameGUI.addChatMessage(str);
    		claySpot = null;
    	}
    	super.onUpdate();
	}
    
    protected void jump()
    {
    	double d = (rand.nextDouble() / 10D);
        motionY = 0.44999998688697815D + d;
        motionX *= 1.4 + d;
        motionZ *= 1.4 + d;
        field_35118_ao = true;
    }
    
    private Vec3D spotClay(){
    	double d = rand.nextDouble() * 10;
    	Vec3D vec3d = getPosition(1.0F);
        Vec3D vec3d1 = getLook(1.0F);
        Vec3D look = vec3d.addVector(vec3d1.xCoord * d, vec3d1.yCoord * d, vec3d1.zCoord * d);
    	int x = MathHelper.floor_double(look.xCoord);
    	int y = MathHelper.floor_double(look.yCoord);
    	int z = MathHelper.floor_double(look.zCoord);
    	
    	for(int i = x - 4; i < x + 4; i++)
    		for(int j = y - 4; j < y + 4; j++)
    			for(int k = z - 4; k < z + 4; k++)
    				if(worldObj.getBlockId(i, j, k) == Block.blockClay.blockID)
    					return Vec3D.createVector(i, j, k);
    	
    	return null;
    }
    
    public ItemStack getChestItem(){
    	return chestItems[rand.nextInt(chestItems.length)].copy();
    }
    
    public boolean canRenovateBuild(){
    	return true;
    }
    
    protected ItemStack getThrownItem(){
    	return new ItemStack(Item.appleRed);
    }
    
    protected int getChunkOffsetX(){
    	return 5;
    }
    
    protected int getChunkOffsetZ(){
    	return 4;
    }
    
    protected ItemStack getHealingItem(){
    	return new ItemStack(Item.appleRed);
    }
    
    public int getMaxHp(){
    	return 30;
    }
    
    public void setTexture(int num){
    	switch(num){
    	case 0:
    		texture = "/mob/BrickBuilder1.png";
    		break;
    	case 1:
    		texture = "/mob/BrickBuilder2.png";
    		break;
    	case 2:
    		texture = "/mob/BrickBuilder3.png"; //contributed by JOJO
			builderFlags |= (1 << 4); //builds twice as fast
			health -= 10;
			tradeGiveNum = 5;
	        tradeReceiveNum = 3;
	        tradeRatio = 6;
    		break;
    	case 3:
    		texture = "/mob/BrickBuilder4.png"; //contributed by JOJO
			builderFlags |= (1 << 5); //pays rent three times as often
			health -= 5;
			tradeGiveNum = 15;
	        tradeReceiveNum = 3;
	        tradeRatio = 2;
    		break;
    	case 4:
    		texture = "/mob/BrickBuilder5.png"; //contributed by JOJO
			builderFlags |= (1 << 6); //heals leader when weak
			health += 5;
			tradeGiveNum = 2;
	        tradeReceiveNum = 2;
	        tradeRatio = 16;
    		break;
    	}
    }
    
    public void initGui(BLDRGui gui){
		super.initGui(gui);
		if(!(currentAction == 0 || currentAction == 7 || currentAction == 9 || currentAction == 10)) return;
		byte byte0 = -16;
        gui.controlList.add(new GuiButton(2, gui.width / 2 - 96, gui.height / 4 + 61 + byte0, 12, 20, "<"));
        gui.controlList.add(new GuiButton(3, gui.width / 2 - 76, gui.height / 4 + 61 + byte0, 12, 20, ">"));
        gui.controlList.add(new GuiButton(4, gui.width / 2 - 100, gui.height / 4 + 86 + byte0, 120, 20, ((builderFlags >> 1) & 1) != 1 && (currentAction == 9 || currentAction == 10) ? "Renovate" : "Don't Renovate"));
	}
    
    public void actionPerformed(BLDRGui gui, GuiButton guibutton){
    	super.actionPerformed(gui, guibutton);
		if(guibutton.id == 2)
        {
			//modify blueNum to previous bluePrint
			blueNum--;
			if(blueNum <= -1)
				blueNum = BLDRBlueprints.singlePrints.size() - 1;
			//if no buildCounts left, add one
			if(buildCount == 0)
				buildCount++;
			//set actionTimer to 0
			actionTimer = 0;
        }
		else if(guibutton.id == 3)
        {
			//modify blueNum to next bluePrint
			blueNum++;
			if(blueNum >= BLDRBlueprints.singlePrints.size())
				blueNum = 0;
			//if no buildCounts left, add one
			if(buildCount == 0)
				buildCount++;
			//set actionTimer to 0
			actionTimer = 0;
        }
		else if(guibutton.id == 4)
        {
			if(currentAction == 9 || currentAction == 10)
				builderFlags ^= (1 << 1);
			else{
				if(currentAction == 0)
					currentAction = 10;
				else if(currentAction == 7)
					currentAction = 9;
				//builderFlags ^= (1 << 1);
			}
            guibutton.displayString = ((builderFlags >> 1) & 1) != 1 ? "Renovate" : "Don't Renovate";
        }
	}
    
    public void drawScreen(int i, int j, float f, BLDRGui gui)
    {
    	super.drawScreen(i, j, f, gui);
    	if(!(currentAction == 0 || currentAction == 7 || currentAction == 9 || currentAction == 10)) return;
    	byte byte0 = -16;
        byte k = 0;
        gui.drawString(gui.fontRenderer, (new StringBuilder()).append("\2476Build Next: \247f").append(blueNum > -1 ? BLDRBlueprints.singlePrintNames.get(blueNum) : "None").toString(), (gui.width / 2 - 100) + k, gui.height / 4 + 46 + byte0, 0xff8d13);        
    }
    
    
    protected Vec3D claySpot;
    
    static final private ItemStack[] chestItems = (new ItemStack[]{
        	new ItemStack(Block.planks, 1), new ItemStack(Block.brick, 1), new ItemStack(Block.brick, 1),
        	new ItemStack(Block.blockClay, 1), new ItemStack(Item.clay, 1), new ItemStack(Item.brick, 1),
        	new ItemStack(Item.coal, 1), new ItemStack(Item.bucketWater, 1), new ItemStack(Item.paper, 1),
        	new ItemStack(Item.bootsChain, 1, 50), new ItemStack(Item.plateLeather, 1, 37), new ItemStack(Item.bed, 1),
        	new ItemStack(Block.wood, 1, 1), new ItemStack(Item.bucketMilk, 1), new ItemStack(Item.clay, 1),
        	new ItemStack(Block.sand, 1), new ItemStack(Item.shovelGold, 1), new ItemStack(Block.stoneOvenIdle, 1),
        	new ItemStack(Item.helmetLeather, 1)
        });

}
