package net.minecraft.src;

public class BLDREntityMultiBuilder extends BLDREntityBuilder{

	public BLDREntityMultiBuilder(World world) {
		super(world);
		buildCount = BLDRBuilderConfig.multiStructureLimit;
		int num = rand.nextInt(3) + 1;
		texture = new StringBuilder("/mob/MultiBuilder").append(num).append(".png").toString();
		health = 30;
		maxWait = 125;
		buildBlock = blockIDs[rand.nextInt(blockIDs.length)];
		if(BLDRBlueprints.singlePrints.size() > 0)
			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
		else
			blueNum = -1;
		while(BLDRBlueprints.singlePrints.size() > 2 && (BLDRBlueprints.isSmallShopIndex(blueNum) || BLDRBlueprints.isSingleBlockIndex(blueNum)))
			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
		
        tradeGiveNum = tradeGiveNum + rand.nextInt(3);
        tradeReceiveNum = rand.nextInt(6);
        tradeRatio = tradeRatio - rand.nextInt(7);
	}
	
	protected void initConvert(int x, String s){
		if(x == Block.workbench.blockID){
			builderFlags |= (1 << 4); //builds twice as fast
			tradeRatio += 5;
		}
		else if(x == Block.ladder.blockID){
			moveSpeed += 0.27D;
			health += 7;
			tradeGiveNum += 7;
		}
    }
	
	public void onDeath(DamageSource d)
    {
		int j = rand.nextInt(3) + 1;
		for(int i = 0; i < j; i++)
			dropItem(blockIDs[rand.nextInt(blockIDs.length)], 1); // drop random blocks
		super.onDeath(d);
    }
	
	public boolean getCanSpawnHere()
    {
		if(worldObj.isDaytime()){
			if(mod_Builders.countBuilders(this.getClass()) >= BLDRBuilderConfig.multiSpawnMaxDay)
				return false;
			else
				return rand.nextInt(100) < BLDRBuilderConfig.multiSpawnRateDay && super.getCanSpawnHere();
		}
		else{
			if(mod_Builders.countBuilders(this.getClass()) >= BLDRBuilderConfig.multiSpawnMaxNight)
				return false;
			else
				return rand.nextInt(100) < BLDRBuilderConfig.multiSpawnRateNight && super.getCanSpawnHere();
		}
    }
	
    public void writeEntityToNBT(NBTTagCompound nbttagcompound)
    {
        super.writeEntityToNBT(nbttagcompound);
    }

    public void readEntityFromNBT(NBTTagCompound nbttagcompound)
    {
        super.readEntityFromNBT(nbttagcompound);
    }
    
    public void onUpdate()
    {
    	if(currentAction >= 1 && currentAction <= 6 && actionTimer == 10099 && ((builderFlags >> 1) & 1) != 1){
    		if(BLDRBlueprints.singlePrints.size() > 0)
    			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
    		else
    			blueNum = -1;
    		while(BLDRBlueprints.singlePrints.size() > 2 && (BLDRBlueprints.isSmallShopIndex(blueNum) || BLDRBlueprints.isSingleBlockIndex(blueNum)))
    			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
    		buildBlock = blockIDs[rand.nextInt(blockIDs.length)];
    	}
    	if(currentAction == 0 && dashTimer == -1 && rand.nextFloat() < 0.001)
    		dash();
    	if(dashTimer > 0)
    		dashTimer--;
    	else if(dashTimer == 0){
    		dashTimer = -1;
    		moveSpeed = 0.8F;
    	}
    	super.onUpdate();
	}
    
    public void dash(){
    	dashTimer = 60 + rand.nextInt(60);
    	moveSpeed = 1.6F;
    }
    
    public ItemStack getChestItem(){
    	return chestItems[rand.nextInt(chestItems.length)].copy();
    }
    
    public boolean canRenovateBuild(){
    	return true;
    }
    
    protected ItemStack getThrownItem(){
    	return new ItemStack(Item.cookie);
    }
    
    protected int getChunkOffsetX(){
    	return 7;
    }
    
    protected int getChunkOffsetZ(){
    	return -7;
    }
    
    protected ItemStack getHealingItem(){
    	return new ItemStack(Item.porkCooked);
    }
    
    public int getMaxHp(){
    	return 30;
    }
    
    public void initGui(BLDRGui gui){
		super.initGui(gui);
		if(!(currentAction == 0 || currentAction == 7 || currentAction == 9 || currentAction == 10)) return;
		byte byte0 = -16;
        gui.controlList.add(new GuiButton(2, gui.width / 2 - 96, gui.height / 4 + 61 + byte0, 12, 20, "<"));
        gui.controlList.add(new GuiButton(3, gui.width / 2 - 76, gui.height / 4 + 61 + byte0, 12, 20, ">"));
        gui.controlList.add(new GuiButton(4, gui.width / 2 - 100, gui.height / 4 + 86 + byte0, 120, 20, ((builderFlags >> 1) & 1) != 1 && (currentAction == 9 || currentAction == 10) ? "Renovate" : "Don't Renovate"));
	}
    
    public void actionPerformed(BLDRGui gui, GuiButton guibutton){
    	super.actionPerformed(gui, guibutton);
		if(guibutton.id == 2)
        {
			//modify blueNum to previous bluePrint
			blueNum--;
			if(blueNum <= -1)
				blueNum = BLDRBlueprints.singlePrints.size() - 1;
			//if no buildCounts left, add one
			if(buildCount == 0)
				buildCount++;
			//set actionTimer to 0
			actionTimer = 0;
        }
		else if(guibutton.id == 3)
        {
			//modify blueNum to next bluePrint
			blueNum++;
			if(blueNum >= BLDRBlueprints.singlePrints.size())
				blueNum = 0;
			//if no buildCounts left, add one
			if(buildCount == 0)
				buildCount++;
			//set actionTimer to 0
			actionTimer = 0;
        }
		else if(guibutton.id == 4)
        {
			if(currentAction == 9 || currentAction == 10)
				builderFlags ^= (1 << 1);
			else{
				if(currentAction == 0)
					currentAction = 10;
				else if(currentAction == 7)
					currentAction = 9;
				//builderFlags ^= (1 << 1);
			}
            guibutton.displayString = ((builderFlags >> 1) & 1) != 1 ? "Renovate" : "Don't Renovate";
        }
	}
    
    public void drawScreen(int i, int j, float f, BLDRGui gui)
    {
    	super.drawScreen(i, j, f, gui);
    	if(!(currentAction == 0 || currentAction == 7 || currentAction == 9 || currentAction == 10)) return;
    	byte byte0 = -16;
        byte k = 0;
        gui.drawString(gui.fontRenderer, (new StringBuilder()).append("\2476Build Next: \247f").append(blueNum > -1 ? BLDRBlueprints.singlePrintNames.get(blueNum) : "None").toString(), (gui.width / 2 - 100) + k, gui.height / 4 + 46 + byte0, 0xff8d13);        
    }
    
    protected int dashTimer;

    static final ItemStack[] chestItems = (new ItemStack[]{
        	new ItemStack(Block.blockClay.blockID, 1, 0), new ItemStack(Block.ladder.blockID, 1, 0), new ItemStack(Block.blockSnow.blockID, 1, 0),
        	new ItemStack(Block.pumpkin.blockID, 1, 0), new ItemStack(Block.wood.blockID, 1, 0), new ItemStack(Block.pumpkinLantern.blockID, 1, 0),
        	new ItemStack(Block.bookShelf.blockID, 1, 0), new ItemStack(Block.cobblestoneMossy.blockID, 1, 0), new ItemStack(Block.wood.blockID, 1, 0),
        	new ItemStack(Block.brick.blockID, 1, 0), new ItemStack(Block.stone.blockID, 1, 0), new ItemStack(Block.dirt.blockID, 1, 0),
        	new ItemStack(Block.obsidian.blockID, 1, 0), new ItemStack(Block.web.blockID, 1, 0), new ItemStack(Block.bookShelf.blockID, 1, 0),
        	new ItemStack(Block.jukebox.blockID, 1, 0), new ItemStack(Block.dispenser.blockID, 1, 0), new ItemStack(Block.chest.blockID, 1, 0),
        	new ItemStack(Block.workbench.blockID, 1, 0), new ItemStack(Block.cloth.blockID, 1, 0), new ItemStack(Block.planks.blockID, 1, 0)
        });
    
    static final int blockIDs[] = new int[]{
    	Block.dirt.blockID, Block.brick.blockID, Block.blockSnow.blockID,
    	Block.cobblestone.blockID, Block.wood.blockID, Block.stone.blockID,
    	Block.stone.blockID, Block.planks.blockID, Block.dirt.blockID,
    	Block.planks.blockID, Block.cloth.blockID, Block.planks.blockID,
    	Block.cobblestoneMossy.blockID, Block.brick.blockID, Block.planks.blockID,
    	Block.sandStone.blockID, Block.sandStone.blockID, Block.stone.blockID,
    	Block.field_35285_bn.blockID, Block.field_35285_bn.blockID //stone brick
    };
    
}
