package net.minecraft.src;

import java.util.ArrayList;
import java.util.List;

/*
 * SandBuilder battle plan: throws cactus at targets if far, and backs up if close to target.
*/ 
public class BLDREntitySandBuilder extends BLDREntityBuilder{

	public BLDREntitySandBuilder(World world) {
		super(world);
		buildCount = BLDRBuilderConfig.sandStructureLimit;
		int rrNum = rand.nextInt(3);
		if(rrNum == 0)
			texture = "/mob/SandBuilder1.png";
		else if(rrNum == 1)
			texture = "/mob/SandBuilder2.png";
		else
			texture = "/mob/SandBuilder3.png";
		health = 15;
		maxWait = 285;	
		buildBlock = BLDRBuilderConfig.sandBlock;
		if(BLDRBlueprints.singlePrints.size() > 0)
			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
		else
			blueNum = -1;
		while(BLDRBlueprints.singlePrints.size() > 2 && (BLDRBlueprints.isSmallShopIndex(blueNum) || BLDRBlueprints.isSingleBlockIndex(blueNum)))
			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());

        tradeGiveNum = 1 + rand.nextInt(4);
        tradeReceiveNum = rand.nextInt(4);
        tradeRatio = 5 + rand.nextInt(12);
        randomStrafe = 0.0F;
        herd = new ArrayList<EntityAnimal>();
        flags = (byte) (rand.nextInt(256) - 128);
        
        randomStrafe = 0.0F;
        moveSpeed = 1.001F;
        cactuses = new EntityItem[6];
	}
	
	protected void initConvert(int x, String s){
		if(x == Block.sand.blockID){
			builderFlags |= (1 << 4); //builds twice as fast
			health += 3;
			flags = 63;
		}
		else if(x == Block.sandStone.blockID){
			builderFlags |= (1 << 4); //builds twice as fast
			health += 6;
			flags = -128;
		}
    }
	
	public void writeEntityToNBT(NBTTagCompound nbttagcompound)
    {
        super.writeEntityToNBT(nbttagcompound);
        nbttagcompound.setByte("Flags", flags);
    }

    public void readEntityFromNBT(NBTTagCompound nbttagcompound)
    {
        super.readEntityFromNBT(nbttagcompound);
        flags = nbttagcompound.getByte("Flags");
    }
	
	public boolean getCanSpawnHere()
    {
		BiomeGenBase biome = worldObj.getWorldChunkManager().getBiomeGenAt((int)posX, (int)posZ);
		if(biome.biomeName != "Desert") return false;
		if(worldObj.isDaytime()){
			if(mod_Builders.countBuilders(this.getClass()) >= BLDRBuilderConfig.sandSpawnMaxDay)
				return false;
			else
				return rand.nextInt(100) < BLDRBuilderConfig.sandSpawnRateDay && super.getCanSpawnHere();
		}
		else{
			if(mod_Builders.countBuilders(this.getClass()) >= BLDRBuilderConfig.sandSpawnMaxNight)
				return false;
			else
				return rand.nextInt(100) < BLDRBuilderConfig.sandSpawnRateNight && super.getCanSpawnHere();
		}
    }
	
	protected void updateEntityActionState()
    {
		super.updateEntityActionState();
		if((flags & 16) == 16 && !noMove){
			moveStrafing = randomStrafe;
    		randomStrafe *= 1.0F - rand.nextFloat() / 40.0F;
		}			
    }
	
	protected int getWeaponID(){
		return Block.cactus.blockID;
	}
	
    public void onUpdate()
    {

    	super.onUpdate();
    	if(worldObj.difficultySetting == 0 && entityliving instanceof EntityPlayer){
    		entityToAttack = null;
    		entityliving = null;
    	}
    	if(updateTimer % 20 == 0 && attackMode == false && entityliving == null){
    		Entity eee = findTargetToAttack();
    		if(attackMode == true)
    			entityToAttack = eee;
    	}
    	if(attackMode == true && entityliving != null){
    		//switch to weapon if not holding it already
    		if(heldObj == null || heldObj.itemID != getWeaponID())
    			heldObj = new ItemStack(getWeaponID(), 1, 0);
    		
    		boolean flag = false;
    		for(int i = 0; i < cactuses.length; i++)
    			if(cactuses[i] != null && handleCactus(cactuses[i]))
    				flag = true;
    		
    		for(int i = 0; i < cactuses.length; i++)
    			if(cactuses[i] != null && cactuses[i].isDead)
    				cactuses[i] = null;
    		
    		if(flag == true)
    			attackTime++;
    		
    		if(withinWeaponRange())
    			swingAtTarget();
    		
    		handleMovement();
    		

    		if(entityliving.isDead == true){
    			entityliving = null;
    		}
    	}
    	else if(attackMode == true && entityliving == null){
    		attackMode = false;
    		heldObj = null;
    		swingTimer = 0;
    		swingProgress = 1F;
    		for(int i = 0; i < cactuses.length; i++)
    			cactuses[i] = null;
    		if(moveSpeed < 0F) moveSpeed = -moveSpeed;
    	}
    	if(currentAction >= 1 && currentAction <= 6 && actionTimer == 10099 && ((builderFlags >> 1) & 1) != 1){
    		if(BLDRBlueprints.singlePrints.size() > 0)
    			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
    		else
    			blueNum = -1;
    		while(BLDRBlueprints.singlePrints.size() > 2 && (BLDRBlueprints.isSmallShopIndex(blueNum) || BLDRBlueprints.isSingleBlockIndex(blueNum)))
    			blueNum = rand.nextInt(BLDRBlueprints.singlePrints.size());
    	}
    	
    	if(!noMove){
    		//determine move speed based on block being stood on; Makes this mob also immune to speed changing effects
        	if(onGround && updateTimer % 40 == 0){
        		determineMoveSpeed();
        		setRandomStrafing();
        		gradualHeal();
        	}
    		
    		//imitates leader
    		if((flags & 64) == 64 && leader instanceof EntityLiving){
    			setFlag(1, leader.isSneaking());
    			isJumping = ((EntityLiving) leader).isJumping;
    		}
    		
    	}
    	
    	if((flags & 15) > 0) handleHerd();
	}
    
    private boolean handleCactus(EntityItem cactus){
    	if(cactus != null){
			if(cactus.isDead){
				cactus = null;
				return false;
			}
			else{
				List list = worldObj.getEntitiesWithinAABBExcludingEntity(cactus, cactus.boundingBox.expand(0.40000000298023224D, 0.4D, 0.40000000298023224D));
		        if(list != null && list.size() > 0)
		        {
		            for(int i = 0; i < list.size(); i++)
		            {
		                Entity entity = (Entity)list.get(i);
		                if(entity instanceof EntityLiving && entity != this)
		                {
		                    entity.attackEntityFrom(DamageSource.func_35525_a(this), 3 + rand.nextInt(5));
		                    cactus.isDead = true;
		                    return true;
		                }
		            }

		        }
			}
			if(cactus.onGround){
				int x = MathHelper.floor_double(cactus.posX);
				int y = MathHelper.floor_double(cactus.boundingBox.minY);
				int z = MathHelper.floor_double(cactus.posZ);
				if(Block.cactus.canPlaceBlockAt(worldObj, x, y, z)){
					cactus.isDead = true;
					worldObj.setBlockWithNotify(x, y, z, Block.cactus.blockID);
				}
			}
	    	return true;
		}
    else
    	return false;
    }
    
    
    private void handleHerd(){
    	//if player cannot see this SandBuilder, spawn animals occasionally
		if(herd.size() < 5 && rand.nextFloat() < 0.02F && !canEntityBeSeen(worldObj.getClosestPlayerToEntity(this, -1))){
			EntityAnimal es = null;
			float f = rand.nextFloat();
			if((flags & 1) == 1 && f < 0.25F)
				es = new EntitySheep(worldObj);
			else if((flags & 2) == 2 && f < 0.5F)
				es = new EntityPig(worldObj);
			else if((flags & 4) == 4 && f < 0.75F)
				es = new EntityCow(worldObj);
			else if((flags & 8) == 8 && f < 1.0F)
				es = new EntityChicken(worldObj);
			
			if(es != null){
				es.setPosition(posX, posY, posZ);
				if(worldObj.entityJoinedWorld(es))
					herd.add(es);
			}
		}
		
		//gather any nearby animals within 8 blocks, but not if another herder within 16 blocks.
		double d = 16D;
		List list = worldObj.getEntitiesWithinAABBExcludingEntity(this, boundingBox.expand(d, d, d));
		ArrayList<EntityAnimal> animalList = new ArrayList<EntityAnimal>();
		boolean otherHerders = false;
		for(int i = 0; i < list.size(); i++)
        {
            Entity entity1 = (Entity)list.get(i);
            if(entity1 instanceof BLDREntitySandBuilder && (((BLDREntitySandBuilder) entity1).flags & 15) > 0){
            	otherHerders = true;
            	break;
            }
            if(!(entity1 instanceof EntityAnimal) || entity1 == ridingEntity || entity1 == riddenByEntity)
			{
				continue;
			}
            animalList.add((EntityAnimal) entity1);
        }
		
		if(otherHerders == false)
			for(int i = 0; i < animalList.size(); i++)
			{
				EntityAnimal entity1 = animalList.get(i);
				double d2 = entity1.getDistanceToEntity(this);
				boolean canGather = false;
				if((entity1 instanceof EntitySheep && (flags & 1) == 1)
						|| (entity1 instanceof EntityPig && (flags & 2) == 2)
						|| (entity1 instanceof EntityCow && (flags & 4) == 4)
						|| (entity1 instanceof EntityChicken && (flags & 8) == 8))
					canGather = true;
					
				if(canGather && (d2 < d / 2F) && entity1.canEntityBeSeen(this) && !herd.contains(entity1))
				{
					herd.add(entity1);
				}
			}
		
		//order herd animals to stay within 6 blocks of SandBuilder
		for(int i = 0; i < herd.size(); i++){
			EntityAnimal es = herd.get(i);
			//remove dead animals from herd
			if(es == null || es.isDead){
				herd.remove(i);
				i--;
				continue;
			}
			//manage animal movement
			float f = es.getDistanceToEntity(this);
			if(f < 6F && es.entityToAttack == this) //wander if close
				es.entityToAttack = null;
			else if(f > 6F && es.entityToAttack != this) //follow if far
				es.entityToAttack = this;
			else if(f > 16F){ //remove from herd if too far
				es.entityToAttack = null;
				herd.remove(i);
				i--;
				continue;
			}
			
			//if animal is idle, make it jump with random variance, change its speed, and make sound
			if(es.onGround && !es.isJumping && !es.hasPath()){
				es.jump();
				es.addVelocity(-0.2D + rand.nextDouble() / 2.5D, -0.1D + rand.nextDouble() / 5D, -0.2D + rand.nextDouble() / 2.5D);
				es.moveSpeed = 0.4F + rand.nextFloat() / 2F;
				es.playLivingSound();
				es.isJumping = true;
			}
		}
		
    }
    
    private void determineMoveSpeed(){
    	if((flags & 32) == 32){
    		int i = MathHelper.floor_double(posX);
    		int j = MathHelper.floor_double(boundingBox.minY);
    		int k = MathHelper.floor_double(posZ);
    		int bid = worldObj.getBlockId(i, j - 1, k);
    		if(bid == Block.sand.blockID || bid == Block.sandStone.blockID)
    			moveSpeed = moveSpeed > 0 ? 1.05F : -1.05F;
    		else
    			moveSpeed = moveSpeed > 0 ? 0.65F : -0.65F;
    	}
    }
    
    private void setRandomStrafing(){
    	if((flags & 16) == 16){
    		randomStrafe = rand.nextFloat() - rand.nextFloat();
    		if(moveSpeed > 1.0F || moveSpeed < -1.0F)
    			randomStrafe *= 1.6F;
    	}
    }
    
    public ItemStack getChestItem(){
    	return chestItems[rand.nextInt(chestItems.length)].copy();
    }
    
    public boolean canRenovateBuild(){
    	return true;
    }
    
    protected ItemStack getThrownItem(){
    	return new ItemStack(Item.bowlEmpty);
    }
    
    private void gradualHeal(){
    	if(flags < 0 && rand.nextFloat() < 0.1F && health < getMaxHp()){
    		health++;
    	}
    }
    
    protected ItemStack getHealingItem(){
    	return new ItemStack(Item.fishRaw);
    }
    
    protected boolean canTossMeat(){
    	return false;
    }
    
    protected int getChunkOffsetX(){
    	return -2;
    }
    
    protected int getChunkOffsetZ(){
    	return 1;
    }
    
    public int getMaxHp(){
    	return 15;
    }
    
    protected Entity findTargetToAttack(){
    	if(currentAction == 0 || currentAction >= 7 && currentAction <= 10){
    		entityliving = getNearestTarget(this, 13D);
    		if(entityliving != null){
    			attackMode = true;
    			actionTimer = 0;
    		}
    		return entityliving;
    	}
    	else
    		return null;
    }
    
    public boolean interact(EntityPlayer entityplayer)
    {
    	if(attackMode == true)
    		return false;
    	else
    		return super.interact(entityplayer);
    }
    
    public boolean attackEntityFrom(DamageSource d, int i)
    {
    	Entity entity = d.func_35526_e();
    	if(entity instanceof EntityLiving && attackMode == false && currentAction == 0 || currentAction >= 7 && currentAction <= 10)
        {
        	
            entityToAttack = entity;
            entityliving = (EntityLiving) entity;
            if(entity instanceof EntityPlayer && currentAction == 7){
            	currentAction = 0;
            	leader = null;
            }
            attackMode = true;
        }
        return super.attackEntityFrom(d, i);
    }
    
    private void swingAtTarget(){
    	if(attackTime > 0) return;
    	swingTimer++;
		prevSwingProgress = (swingTimer) * 0.125F;
		swingProgress = (swingTimer) * 0.125F;
		//once swing completed, use weapon
		if(swingTimer == 8){
			for(int i = 0; i < cactuses.length; i++)
				if(i == 0 || this.getDistanceToEntity(entityliving) > 11D && rand.nextInt(4) == 0)
					useWeapon(i, i == 0 ? 1.0F : 1 + rand.nextFloat() * 6);
			swingTimer = 0;
		}
    }
    
    private void useWeapon(int i, float f){
    	if(getWeaponID() == Block.cactus.blockID){
    		double d = entityliving.posX - posX;
            double d1 = entityliving.posZ - posZ;
            if(attackTime == 0)
            {
            	float cactusSpeed = 1 + rand.nextFloat();
            	EntityItem cactus = new EntityItem(worldObj, posX, posY + 1.2D, posZ, new ItemStack(Block.cactus));
                double d2 = entityliving.posY - 0.20000000298023224D - cactus.posY;
                float f1 = MathHelper.sqrt_double(d * d + d1 * d1) * 0.3F;
                worldObj.entityJoinedWorld(cactus);
                setCactusHeading(cactus, d, (d2 + (double)f1), d1, 0.66F * cactusSpeed, 4F * f);
                attackTime = 15 + rand.nextInt(50);
                cactus.age = 5900;
                cactus.delayBeforeCanPickup = 200;
                cactuses[i] = cactus;
            }
            rotationYaw = (float)((Math.atan2(d1, d) * 180D) / 3.1415927410125732D) - 90F;
    	}
    }
    
    public void setCactusHeading(EntityItem cactus, double d, double d1, double d2, float f, 
            float f1)
    {
        float f2 = MathHelper.sqrt_double(d * d + d1 * d1 + d2 * d2);
        d /= f2;
        d1 /= f2;
        d2 /= f2;
        d += rand.nextGaussian() * 0.0074999998323619366D * (double)f1;
        d1 += rand.nextGaussian() * 0.0074999998323619366D * (double)f1;
        d2 += rand.nextGaussian() * 0.0074999998323619366D * (double)f1;
        d *= f;
        d1 *= f;
        d2 *= f;
        cactus.motionX = d;
        cactus.motionY = d1;
        cactus.motionZ = d2;
    }
    
    private boolean withinWeaponRange(){
    	if(getWeaponID() == Block.cactus.blockID){
    		return this.getDistanceToEntity(entityliving) < 20D;
    	}
    	else
    		return true;
    }
    
    protected void handleMovement(){
    	if(this.getDistanceToEntity(entityliving) < 9D && moveSpeed > 0F || this.getDistanceToEntity(entityliving) > 14.5D && moveSpeed < 0F)
    		moveSpeed = -moveSpeed;
    	
    	if(this.isCollidedHorizontally && moveSpeed < 0.0F){
    		if(onGround) jump();
    		else if(!onGround && motionY < 0.0D) wallJump();
    	}
    }
    
    protected void wallJump(){
    	motionY = 0.56D;
    	this.moveFlying(0.2F * rand.nextFloat() - 0.1F, 1.27F, 1F);
    }
    
    protected void selectIdleAction(){
    	if(!attackMode)
    		super.selectIdleAction();
    }
    
    public void playerEntityAttack(Entity e){
    	if(e == this) return;
    	
    	if(e instanceof EntityLiving){
    		entityliving = (EntityLiving)e;
    		attackMode = true;
    		actionTimer = 0;
    		entityToAttack = e;
    	}
    }
    
    public void initGui(BLDRGui gui){
		super.initGui(gui);
		if(!(currentAction == 0 || currentAction == 7 || currentAction == 9 || currentAction == 10)) return;
		byte byte0 = -16;
        gui.controlList.add(new GuiButton(2, gui.width / 2 - 96, gui.height / 4 + 61 + byte0, 12, 20, "<"));
        gui.controlList.add(new GuiButton(3, gui.width / 2 - 76, gui.height / 4 + 61 + byte0, 12, 20, ">"));
        gui.controlList.add(new GuiButton(4, gui.width / 2 - 100, gui.height / 4 + 86 + byte0, 120, 20, ((builderFlags >> 1) & 1) != 1 && (currentAction == 9 || currentAction == 10) ? "Renovate" : "Don't Renovate"));
	}
    
    public void actionPerformed(BLDRGui gui, GuiButton guibutton){
    	super.actionPerformed(gui, guibutton);
		if(guibutton.id == 2)
        {
			//modify blueNum to previous bluePrint
			blueNum--;
			if(blueNum <= -1)
				blueNum = BLDRBlueprints.singlePrints.size() - 1;
			//if no buildCounts left, add one
			if(buildCount == 0)
				buildCount++;
			//set actionTimer to 0
			actionTimer = 0;
        }
		else if(guibutton.id == 3)
        {
			//modify blueNum to next bluePrint
			blueNum++;
			if(blueNum >= BLDRBlueprints.singlePrints.size())
				blueNum = 0;
			//if no buildCounts left, add one
			if(buildCount == 0)
				buildCount++;
			//set actionTimer to 0
			actionTimer = 0;
        }
		else if(guibutton.id == 4)
        {
			if(currentAction == 9 || currentAction == 10)
				builderFlags ^= (1 << 1);
			else{
				if(currentAction == 0)
					currentAction = 10;
				else if(currentAction == 7)
					currentAction = 9;
				//builderFlags ^= (1 << 1);
			}
            guibutton.displayString = ((builderFlags >> 1) & 1) != 1 ? "Renovate" : "Don't Renovate";
        }
	}
    
    public void drawScreen(int i, int j, float f, BLDRGui gui)
    {
    	super.drawScreen(i, j, f, gui);
    	if(!(currentAction == 0 || currentAction == 7 || currentAction == 9 || currentAction == 10)) return;
    	byte byte0 = -16;
        byte k = 0;
        gui.drawString(gui.fontRenderer, (new StringBuilder()).append("\2476Build Next: \247f").append(blueNum > -1 ? BLDRBlueprints.singlePrintNames.get(blueNum) : "None").toString(), (gui.width / 2 - 100) + k, gui.height / 4 + 46 + byte0, 0xff8d13);        
    }
    
    /*private byte setAction(){
    	switch (currentAction){
    	case 0:
    		if(actionTimer < 200 + rand.nextInt(1000)) return 0;
    		else if(moveSpeed > 1.0F) return (byte) (rand.nextInt(2) + 1);
    	case 1:
    		if(actionTimer < 200 + rand.nextInt(1000)) return 0;
    		else return 3;
    	case 2:
    		if(actionTimer < 200 + rand.nextInt(1000)) return 0;
    		else return 3;
    	case 3:
    		return -((xDif - zDif) / 2F);
    	default:
    		return currentAction;
    	}
    }*/
    
    protected boolean attackMode;
    protected EntityLiving entityliving; //Enemy mob
    protected int swingTimer; //timer for swinging weapon in combat
    protected EntityItem cactuses[];
	
    static final private ItemStack[] chestItems = (new ItemStack[]{
        	new ItemStack(Block.sand, 6), new ItemStack(Block.sand, 4), new ItemStack(Block.sand, 2),
        	new ItemStack(Block.blockClay, 1), new ItemStack(Item.clay, 1), new ItemStack(Item.bone, 1),
        	new ItemStack(Block.cloth, 2, 8), new ItemStack(Item.arrow, 2), new ItemStack(Item.bone, 2),
        	new ItemStack(Block.stoneOvenIdle, 1), new ItemStack(Item.pocketSundial, 1), new ItemStack(Item.stick, 3),
        	new ItemStack(Block.lever, 1), new ItemStack(Item.minecartEmpty, 1), new ItemStack(Item.coal, 3),
        	new ItemStack(Item.stick, 1), new ItemStack(Item.bucketWater, 1), new ItemStack(Item.swordGold, 1),
        	new ItemStack(Block.cactus, 1), new ItemStack(Block.sandStone, 3), new ItemStack(Block.stoneOvenIdle, 1),
        	new ItemStack(Block.sandStone, 3)
        });
    
    private float randomStrafe;
    protected ArrayList<EntityAnimal> herd;
    
    /** -flags-
     * bit 1: Herds Sheep
     * bit 2: Herds Pigs
     * bit 3: Herds Cows
     * bit 4: Herds Chickens
     * bit 5: Randomly strafes
     * bit 6: Changes speed based on terrain
     * bit 7: Imitates leader behavior
     * bit 8: Gradually heals
     */
    protected byte flags;
    
}
